﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_parse_file_sizes
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Convert a file size into bytes.
        private void btnParse_Click(object sender, EventArgs e)
        {
            double bytes = ParseFileSize(txtSize.Text, 1024);
            txtBytes.Text = bytes.ToString("N0");
            txtCheck.Text = ToFileSize(bytes);
        }

        // Parse a file size.
        private string[] SizeSuffixes = { "bytes", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB" };
        private double ParseFileSize(string value, int kb_value)
        {
            // Remove leading and trailing spaces.
            value = value.Trim();

            try
            {
                // Find the last non-alphabetic character.
                int ext_start = 0;
                for (int i = value.Length - 1; i >= 0; i--)
                {
                    // Stop if we find something other than a letter.
                    if (!char.IsLetter(value, i))
                    {
                        ext_start = i + 1;
                        break;
                    }
                }

                // Get the numeric part.
                double number = double.Parse(value.Substring(0, ext_start));

                // Get the extension.
                string suffix;
                if (ext_start < value.Length)
                {
                    suffix = value.Substring(ext_start).Trim().ToUpper();
                    if (suffix == "BYTES") suffix = "bytes";
                }
                else
                {
                    suffix = "bytes";
                }

                // Find the extension in the list.
                int suffix_index = -1;
                for (int i = 0; i < SizeSuffixes.Length; i++)
                {
                    if (SizeSuffixes[i] == suffix)
                    {
                        suffix_index = i;
                        break;
                    }
                }
                if (suffix_index < 0)
                    throw new FormatException("Unknown file size extension " + suffix + ".");

                // Return the result.
                return Math.Round(number * Math.Pow(kb_value, suffix_index));
            }
            catch (Exception ex)
            {
                throw new FormatException("Invalid file size format", ex);
            }
        }

#region Validation

        // Return a string describing the value as a file size.
        // For example, 1.23 MB.
        public string ToFileSize(double value)
        {
            string[] suffixes = { "bytes", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB" };
            for (int i = 0; i < suffixes.Length; i++)
            {
                if (value <= (Math.Pow(1024, i + 1)))
                {
                    return ThreeNonZeroDigits(value / Math.Pow(1024, i)) + " " + suffixes[i];
                }
            }

            return ThreeNonZeroDigits(value / Math.Pow(1024, suffixes.Length - 1)) +
                " " + suffixes[suffixes.Length - 1];
        }

        // Return the value formatted to include at most three
        // non-zero digits and at most two digits after the
        // decimal point. Examples:
        //         1
        //       123
        //        12.3
        //         1.23
        //         0.12
        private static string ThreeNonZeroDigits(double value)
        {
            if (value >= 100)
            {
                // No digits after the decimal.
                return value.ToString("0,0");
            }
            else if (value >= 10)
            {
                // One digit after the decimal.
                return value.ToString("0.0");
            }
            else
            {
                // Two digits after the decimal.
                return value.ToString("0.00");
            }
        }

#endregion Validation

    }
}
